/*!

\mainpage PROPS

\section intro Introduction

This is the technical documentation for PROPS, generated from the source code
by Doxygen. For a more didactic and general introduction, see the
<a href="http://select.cs.cmu.edu/code/sntools/index.html">PROPS main page</a>.

\section install Installation

If all the dependencies are installed (Python 2.6, IT++, SWIG, numpy),
typing 'make' or 'make python' should build PROPS and its Python wrapper code.
Typing 'make test' should build all the object files and C++ tests.
Typing 'make dist' will build a tarball suitable for distribution.
Typing 'python test.py' will run a series of unit tests.

\section over Overview

\section libraries Libraries

<a href="http://itpp.sourceforge.net/">IT++</a> is employed for linear algebra
and matrix manipulation. In order to efficiently construct numpy matrices out of
IT++ matrices, it helps to know how IT++ stores matrices internally. There does
not appear to be clear documentation on this, but PROPs assumes a dense column-major
format which is apparently correct.

<a href="http://www.swig.org">SWIG</a> is used to
generate python wrappers for the C++ code.

<a href="http://numpy.scipy.org">numpy</a> is used in all Python code for matrix
manipulation. Matrices are transferred from IT++ to numpy with the help of SWIG
and numpy's C interface.

\section style Coding Style

The basic coding style is taken from the
<a href="http://google-styleguide.googlecode.com/svn/trunk/cppguide.xml">
Google C++ style guide</a>. In particular, operator overloading is used only
where it is obvious what the operators should do, and constructors are only
used where there is an obvious choice of initialization parameters;
appropriately named static methods are used in other cases. Exceptions are used
to signal errors, which automatically become Python exceptions under SWIG.

All code is documented with doxygen. Method descriptions are placed in header
files (except for static methods). Documentation of static methods is included
for work on internals, even though those methods are not accessible from
Python.

*/
