/*! \file dist.h
 *  \brief Definition of Func class for (explicit) distributions on permutations
 */

#ifndef DIST_H
#define DIST_H

#include "func.h"
#include "partition.h"
//#include "matrix.h"

/// Construct the uniform distribution on S_n
Func *uniform(int n);

/// Construct the k-subset mixing model
Func *mix_subset(int n, int k, Permutation::perm_val_t *subset);

/// Construct the pairwise mixing model
Func *mix_pair(int n, Permutation::perm_val_t i,
                      Permutation::perm_val_t j, Func::val_t p);

/// Construct the insertion mixing model
Func *insertion_mix(int n);

/// The Mallows model
/*! The distribution defined by P(p) = exp(-c * d(p, p0))
 *   where d is the Kendall's tau distance: the number of adjacent
 *   swaps to transform p^-1 into p0^-1
 */
Func *mallows(Permutation p0, double c);



#endif
