/*! \file obs.cpp
 *  \brief Implementation of various observation models
 */

#include "obs.h"
#include "util.h"

Func *multitrack_obs(int n, int k, int *tracks, int *obs, Func::val_t pi)
{
        Func *f = new Func(n);

        int j;
        for (int i = 0; i < f->funclen; i++) {
                Permutation p = Permutation::from_index(n, i);
                // see if this permutation fits the observations
                for (j = 0; j < k; j++)
                        if (p(tracks[j]) != obs[j])
                                break;
                if (j == k)
                        f->func[i] = pi;
                else
                        f->func[i] = (1-pi) / (fact(n)/fact(n-k) - 1);
        }

        return f;
}

Func *singletrack_obs(int n, int i, int j, Func::val_t pi)
{
        return multitrack_obs(n, 1, &i, &j, pi);
}

Func *bluetooth_obs(int n, int k, int *tracks, int *obs, Func::val_t pi)
{
        Func *f = new Func(n);

        for (int i = 0; i < f->funclen; i++) {
                Permutation p = Permutation::from_index(n, i);
                // see if this permutation fits the observations
                if (p.setmapsto(k, tracks, obs))
                        f->func[i] = pi;
                else
                        f->func[i] = (1-pi) / (fact(n)/fact(k)/fact(n-k) - 1);
        }

        return f;
}

Func *pair_rank_obs(int n, int i, int j, Func::val_t pi)
{
        Func *f = new Func(n);

        for (int k = 0; k < f->funclen; k++) {
                Permutation p = Permutation::from_index(n, k);
                bool agrees = false;

                for (int m = 0; m < n; m++) {
                        if (p(m) == i) {
                                agrees = true;
                                break;
                        } else if (p(m) == j) {
                                agrees = false;
                                break;
                        }
                }

                if (agrees)
                        f->func[k] = pi;
                else
                        f->func[k] = 1 - pi;
        }

        return f;
}
